@@ -25,15 +25,7 @@
 /* Must be before all other gnome includes!! */
 #include <glib/gi18n-lib.h>
 
-#include <gtk/gtksignal.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkvseparator.h>
-#include <gtk/gtkfontsel.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkstock.h>
+#include <gtk/gtk.h>
 
 #include "gnome-stock-icons.h"
 #include "gnometypebuiltins.h"
@@ -965,7 +957,8 @@ gnome_font_picker_dialog_destroy(GtkWidget *widget,
     gfp->_priv->font_dialog = NULL;
 } /* gnome_font_picker_dialog_destroy */
 
-GtkWidget *gnome_font_picker_create_inside(GnomeFontPicker *gfp)
+static GtkWidget *
+gnome_font_picker_create_inside(GnomeFontPicker *gfp)
 {
     GtkWidget *widget;
 
@@ -1017,7 +1010,7 @@ gnome_font_picker_label_use_font_in_label  (GnomeFontPicker *gfp)
 	gtk_widget_modify_font (gfp->_priv->font_label, desc);
 
 	pango_font_description_free (desc);
-	
+
 } /* gnome_font_picker_set_label_font */
 
 static void
